from turbogears import identity, expose, redirect, flash
from turbogears import widgets, validators, error_handler, validate
from sqlobject.sqlbuilder import *

class NewControllerFields(widgets.WidgetsList):
    controllername = widgets.TextField(label="Controllername:",validator=validators.NotEmpty, attrs=[('id','controllernamefield')])
    description = widgets.TextArea(label = "Beschreibung:",  default="kurze Beschreibung des Controllers",  rows=5, cols=40, attrs=[('id','descriptionfield')])

class NewControllerInsanceFields(widgets.WidgetsList):
    from vlab.model import VlabController
    controller_options = [(controller.id, controller.controller_name)
                                        for controller in VlabController.select()]

    controllername = widgets.TextField(label="Controllername:",validator=validators.NotEmpty, attrs=[('id','controllername_field')])
    controller_typ = widgets.SingleSelectField(label="Controller-Typ:", options = controller_options, attrs=[('id','controllertpye_field')], validator=validators.String() )
    cmdreset = widgets.TextField(label="Cmd Reset:",validator=validators.NotEmpty, attrs=[('id','cmdreset_field')])
    cmdenv = widgets.TextArea(label = "Cmd Initialisierung:",  default="",  rows=5, cols=40, attrs=[('id','cmdenv_field')])
    envuser = widgets.TextField(label="Benutzername:",validator=validators.NotEmpty, attrs=[('id','envuser_field')])
    envpw = widgets.PasswordField(label="Passwort:",validator=validators.NotEmpty, attrs=[('id','envpw_field')])

class ControllerDetailFields(widgets.WidgetsList):
    controllername = widgets.TextField(label="Controllername:",validator=validators.NotEmpty, attrs=[('id','mcontrollername_field')])
   # controller_typ = widgets.SingleSelectField(label="Controller-Typ:", options = controller_options, attrs=[('id','controllertpye_field')], validator=validators.String() )
    cmdreset = widgets.TextField(label="Cmd Reset:",validator=validators.NotEmpty, attrs=[('id','mcmdreset_field')])
    cmdenv = widgets.TextArea(label = "Cmd Initialisierung:",  default="",  rows=5, cols=40, attrs=[('id','mcmdenv_field')])
    envuser = widgets.TextField(label="Benutzername:",validator=validators.NotEmpty, attrs=[('id','menvuser_field')])
    envpw = widgets.PasswordField(label="Passwort:",validator=validators.NotEmpty, attrs=[('id','menvpw_field')])


class ControllerListItem:

    def __init__(self, name, description):
        self.name = name
        self.description = description

newcontroller_form = widgets.TableForm(fields = NewControllerFields(), submit_text="speichern")
newcontrollerinstance_form = widgets.TableForm(fields = NewControllerInsanceFields(), submit_text="speichern")

class MicroControllerController(identity.SecureResource):
    require = identity.not_anonymous()

    @expose(template="vlab.templates.controller.index")
    def index(self):
        from vlab.model import VlabController

        # if user is admin -> redirect to managment interface
        if identity.in_group("admin"):
            raise redirect('/controller/managecontroller')


        controllerdatagrid = widgets.DataGrid(fields=[
                                        ('Controller', 'name'),
                                        ('Beschreibung','description'),
                                ])

        controllers = [ControllerListItem(c.controller_name, c.description) for c in VlabController.select()]

        return dict(controllerdatagrid =controllerdatagrid, controllerlist = controllers)

    @expose('json')
    def setcontrollerused(self, controller_id):
        from vlab.model import VlabControllerInUse, VlabControllerInstance

        error = ""

        controller = VlabControllerInstance.get(controller_id)
        controllerused = VlabControllerInUse.selectBy(controller = controller)

        if controllerused == []:
            VlabControllerInUse(controller = controller, user = identity.current.user)
        else:
            # pruefen, ob user noch online
            error = "Controller wird bereits genutzt"

        return dict(error = error)



    @expose('json')
    def getavailablecontrollers(self, controller_id):
        from vlab.model import VlabControllerInstance, VlabControllerInUse, VlabReservation, VlabController
        import datetime, time

        date_from = date_to = datetime.datetime.now()

        # select all controller instances of controllertype

        ss1 = Select([VlabControllerInUse.q.controllerID])


        ss0 = Select([VlabControllerInstance.q.id], AND(
                                                                    (VlabControllerInstance.q.vlab_controllerID == controller_id),
                                                                    NOTIN(VlabControllerInstance.q.id, ss1)
                                                                    ))

        availablecontrollers = [(c.id, c.controller_name) for c in VlabControllerInstance.select(IN(VlabControllerInstance.q.id,ss0))]

        return dict(controllers = availablecontrollers)


    @expose('json')
    def iscontrolleravailable(self, controller_id):
        from vlab.model import VlabControllerInUse, VlabControllerInstance
        from datetime import datetime
        import cherrypy

        cherrypy.session['user'][identity.current.user] = "online"

        print "XXXXXXXXXX"

        print list(cherrypy.session.get('user',None))

        print "XXXXXXXXXX"

        free = 0
        until = "21.05.07 17:15"
        servertime = datetime.now()
        entry = VlabControllerInUse.selectBy(controller = VlabControllerInstance.get(controller_id))

        if list(entry) != []:
            if entry.user == identity.current.user:
                free = 1

        if free == 1:
            #get next reservation
            where_clause = AND((VlabReservation.q.fromtime <= func.now()), (VlabReservation.q.totime >= func.now()), (VlabReservation.q.controllerID == controller_id))
            reservations = VlabReservation.select(where_clause)

            if reservations == []:
                until = "keine Reservierung vorhanden"
            else:
                until = reservations[0].fromtime
            print "XXXXXXXX"
            print reservations
            print entry
            print until
            print servertime

        return dict(free = free, until = until, servertime = servertime)


    @identity.require(identity.in_group("admin"))
    @expose(template="vlab.templates.controller.newcontroller")
    def managecontroller(self):
        from vlab.model import VlabController



        controller_options = [(controller.id, controller.controller_name)
                                        for controller in VlabController.select()]

        controllerfieldset = widgets.FieldSet(legend="Controller: ",
                                        fields = [widgets.SingleSelectField('controllerselectfield',
                                                                                                options = controller_options,
                                                                                                attrs=[('size','20'),('id','controllerselectfield')],
                                                                                                label='',
                                                                                                field_class=('ProjectSelectField'),
                                                                                                validator = validators.Any),
                                                     widgets.ImageButton(label = '', src = "/static/images/cancel.png", attrs = [('id','submit_delete_controller')])

                                                    ]
                                        )

        controllerinsfieldset = widgets.FieldSet(legend="Controller Instanzen: ",
                                        fields = [widgets.SingleSelectField('controllerinsselectfield',
                                                                                                options = [],
                                                                                                attrs=[('size','20'),('id','controllerinstanceselectfield')],
                                                                                                label='',
                                                                                                field_class=('ProjectSelectField'),
                                                                                                validator = validators.Any),
                                                     widgets.ImageButton(label = '', src = "/static/images/cancel.png", attrs = [('id','submit_delete_controllerinstance')])

                                                    ]
                                        )

        detailsfieldset = widgets.FieldSet(legend="Details: ",
                                        fields = ControllerDetailFields()
                                        )


        tabber = widgets.Tabber()
        return dict(tabber = tabber,
                            newcontroller = newcontroller_form, newcontroller_action = '/controller/newcontroller',
                            newcontrollerinstance = newcontrollerinstance_form, newcontrollerinstance_action = '/controller/newcontrollerinstance',
                            controllerfieldset=controllerfieldset, controllerinsfieldset=controllerinsfieldset, detailsfieldset = detailsfieldset)

    @expose('json')
    def getcontrollerinstances(self, controller_id):
        from vlab.model import VlabControllerInstance, VlabController
        msg = ""
        controller_instances = []
        try:
            controller_instances = [(c.id, c.controller_name) for c in VlabControllerInstance.selectBy(vlab_controller = VlabController.get(controller_id))]
        except:
            msg = "Es ist ein Fehler beim Abrufen der Controller-instanzen aufgetreten"

        return dict(msg = msg, controller_instances = controller_instances)

    @expose('json')
    def getcontrollerinstancedetails(self, ci_id):
        from vlab.model import VlabControllerInstance
        msg = ""

        ci = ""
        try:

            ci =  VlabControllerInstance.get(ci_id)
            controller_name = ci.controller_name
            cmdreset = ci.cmdreset
            cmdenv = ci.cmdenv
            envuser = ci.envuser
            envpw = ci.envpw

        except:
            msg = "Es ist ein Fehler beim Abrufen der Controller-Instanze-Details aufgetreten"

        return dict(msg = msg, controller_name = controller_name, cmdenv = cmdenv, cmdreset = cmdreset, envuser = envuser, envpw = envpw)

    @identity.require(identity.in_group("admin"))
    @expose('json')
    def deletecontroller(self, controller_id):
        from vlab.model import VlabController, VlabControllerInstance

        msg = ""

        try:
            c = VlabController.get(controller_id)
            for cis in list(VlabControllerInstance.selectBy(vlab_controller = c)):
                deletecontrollerinstance(ci_id = cis.id)

            c.destroySelf()
            #msg = "Controller mit allen Instanzen erfolgreich geloescht."

        except:
            msg = "Fehler beim Loeschen des Controllers"

        return dict(msg = msg)


    @identity.require(identity.in_group("admin"))
    @expose('json')
    def deletecontrollerinstance(self, ci_id):
        from vlab.model import  VlabControllerInstance
        msg = ""
        try:
            c = VlabControllerInstance.get(ci_id)
            c.destroySelf()
           # msg = "Controller-Instanz erfolgreich geloescht."
        except:
            msg = "Fehler beim Loeschen der Controller-Instanz."

        return dict(msg = msg)


    @expose()
    @identity.require(identity.in_group("admin"))
    @validate(form = newcontroller_form)
    @error_handler(managecontroller)
    def newcontroller(self, *args, **kwargs):
        from vlab.model import VlabController
        import cherrypy, os
        controller = VlabController.selectBy(controller_name = kwargs['controllername'])

        if list(controller) != []:
            flash('Neuer Controller-Typ: Es existiert bereits ein Controller mit gleichen Namen (' + kwargs['controllername']+')')
        else:
            try:
                VlabController(controller_name = kwargs['controllername'], description =kwargs['description'] )
            except:
                flash('Fehler beim ERstellen eines neuen Controllers')
            # create demo dir
        raise redirect('/controller/managecontroller')

    @expose()
    @identity.require(identity.in_group("admin"))
    @validate(form = newcontrollerinstance_form)
    @error_handler(managecontroller)
    def newcontrollerinstance(self, *args, **kwargs):
        from vlab.model import VlabControllerInstance, VlabController
        print kwargs
        controller_typ = VlabController.get(kwargs['controller_typ'])

        controller_ins = VlabControllerInstance.selectBy(controller_name = kwargs['controllername'])
        if list(controller_ins) != []:
            flash('Neue Controller-Instanz: Es existiert bereits eine Controller-Instanz mit gleichen Namen (' + kwargs['controllername']+')')
        else:
            VlabControllerInstance(vlab_controller = controller_typ, controller_name = kwargs['controllername'],
                                                cmdreset = kwargs['cmdreset'], cmdenv = kwargs['cmdenv'],
                                                envuser = kwargs['envuser'], envpw = kwargs['envpw'])


        raise redirect('/controller/managecontroller')
