#!/usr/bin/python
# -*- coding: latin-1 -*-

# Copyright (c) 2008 by Bernd Kalwar
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA

from turbogears import controllers, expose, widgets, flash, validators, redirect, identity, error_handler, validate
from sqlobject import *
from kid import Element
from vlab.model import User, UserProject
import cherrypy, os

PROJECT_DIR= cherrypy.config.get("vlab.projectdir");

class IdeSubController(controllers.Controller):

    @expose()
    def index(self):
        return dict()


    ##
    # open a file and return value
    # parameter:
    #   - project_id: id of the project
    #   - fielname: name of the file to save
    ##
    @expose('json')
    def openfile_deprecated(self, project_id, filename):

        msg = ""
        value = ""
        if project_id == "" or filename == "":
            # return error message
            msg = _(u"Fehler: Kann Projekt oder Dateiname nicht identifizieren.")
        else:
            try:
                target_file_name =  os.path.join(PROJECT_DIR, str(identity.current.user.display_name),str(project_id), filename)
                print "-------------"
                print target_file_name
                f=open(target_file_name, 'rb')

                value = f.read()
            except:
                msg = _(u"Fehler: Fehler beim Lesen der Datei.")

        return dict(filename = filename, value = value, project_id = project_id, msg=msg)


    ##
    # saves file
    # parameter:
    #   - project_id: id of the project
    #   - fielname: name of the file to save
    #   - value: Value of the file
    ##
    @expose('json')
    @identity.require(identity.Any(identity.not_anonymous()))
    def savefile(self, *args, **kwargs) :
        msg = ""
        error = False;
        try:
            project = UserProject.get(kwargs["projectid"])
            if (project.user == identity.current.user) or ("admin" in identity.current.groups):

                target_file_name =  os.path.join(PROJECT_DIR, project.user.display_name,str(project.id), kwargs["filename"])
                target_dir = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))

                try:
                    if not os.path.exists(target_dir):
                            os.makedirs(target_dir)
                    f=open(target_file_name, 'w')
                    f.write(kwargs["value"])
                    f.close()
                except:
                    msg = "Fehler: Fehler beim schreiben in Datei."
                    error = True
            else:
                msg = "Fehler: Sie besitzten nicht die benoetigten Rechte um in Datei zu schreiben."
                error = True
        except:
            msg = "Fehler: Fehler beim schreiben in Datei."
            error = True

        return dict(msg = msg, error = error)

    @expose('json')
    def getprojectfiles(self, *args, **kwargs):
        files = []
        try:
            project = UserProject.get(kwargs["projectid"])
            target_dir = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))
            if not os.path.exists(target_dir):
                os.makedirs(target_dir)

            for root, dir, file in os.walk(target_dir, topdown=True):
                if root == target_dir and len(file) != 0:
                    files.append(file)

        except:
            files = []

        return dict(files = files)

    ##
    # open a file and return value
    # parameter:
    #   - project_id: id of the project
    #   - fielname: name of the file to save
    ##
    @expose('json')
    def openfile(self, *args, **kwargs):

        file = "";
        error = False;
        try:
            project = UserProject.get(kwargs["projectid"])
            target_file_name = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id),kwargs["filename"])
            f=open(target_file_name, 'r')
            file = f.read()
            f.close()

        except:
            error = True


        return dict(error = error, file = file)

    @expose('json')
    @identity.require(identity.Any(identity.not_anonymous()))
    def deletefile(self, *args, **kwargs):

        msg = "";
        error = False;

        try:

            project = UserProject.get(kwargs["projectid"])
            if (project.user == identity.current.user) or ("admin" in identity.current.groups):
                target_file_name = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id),kwargs["filename"])
                os.remove(target_file_name)
            else:
                error = True
                msg = _(u"Sie besitzen nicht die ntigen Rechte, um eine Datei zu loeschen")
        except:
            error = True
            msg = _(u"Es ist ein Fehler beim Lschen der Datei aufgetreten")

        return dict(error = error, msg = msg)




    @expose('json')
    def build(self,*args, **kwargs):
        import sys
        error = False
        msg = ""
        value = "Error"
        try:
            project = UserProject.get(kwargs["projectid"])
            if (project.user == identity.current.user) or ("admin" in identity.current.groups):

                target = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))
                os.chdir(target)

                cmd = "make";

                stdin, stdout = os.popen4(cmd)
                value = stdout.read()
            else:
                msg = _(u"Sie haben nicht die ntigen Berechtigungen, dieses Projekt zu kompilieren.")
                error = True
        except:
            msg = _(u"Fehler beim Kompilieren des Projekts")
            error = True

        return dict(value = value, error = error, msg = msg)


    @expose(template="vlab.templates.projectsubcontroller.terminal")
    def debug(self):
        return dict()



    @expose()
    def terminal(self, w= None, h = None, c = 0, k = None):
        import time
        from terminal import Terminal, Multiplex, SynchronizedMethod
        import cherrypy
        from vlab.model import MicroController
        #from vlab.model import VlabControllerInstance


        if cherrypy.session.get('destroyterminal',False) == True:
            dump = "<h1>Ihre Nutzzeit ist abgelaufen</h1>"

        else:

            if cherrypy.session.get('destroyterm', False) == True:
                cherrypy.session['term'] = None
                cherrypy.session['destroyterm'] = False

            if cherrypy.session.get('term') == None:
                if not (w>2 and w<256 and h>2 and h<100):
                    w,h=80,25

                if cherrypy.session.get('controllerid',-1) != -1:
                    mc = MicroController.get(cherrypy.session.get('controllerid',-1))

                    cmd = "su -l " + mc.linux_user

                    multi  = Multiplex(cmd)
                    cherrypy.session['multi'] = multi
                    term  =multi.create(w,h)
                    cherrypy.session['term'] = term
                    # wait 1 second for password prompt
                    time.sleep(1)
                    password = mc.linux_pw
                    for l in password:
                        multi.proc_write(term,l)
                    multi.proc_write(term,'\n')
                    time.sleep(1)
                    multi.proc_write(term,'clear\n')
            else:
                term = cherrypy.session.get('term')
                multi = cherrypy.session.get('multi')

            if k:
                multi.proc_write(term,k)

            time.sleep(0.002)
            dump=multi.dump(term,c)


            # set content type fuer xmlhttprequest
            cherrypy.response.headers["Content-Type"] = "text/xml"

        return dump

    @expose('json')
    def closeterminal(self):
        import cherrypy
        print "CLOSE TERM"
        if cherrypy.session.get('term') != None:
            #cherrypy.session.get('term') = None
            pass

        return dict()
