#!/usr/bin/python
# -*- coding: latin-1 -*-

# Copyright (c) 2008 by Bernd Kalwar
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA

from turbogears import controllers, expose, widgets, flash, validators, redirect, identity, error_handler, validate
from sqlobject import *
from kid import Element
from vlab.model import User, UserProject, MicroController, Reservation, MicroControllerUse
from sqlobject import *
from sqlobject.sqlbuilder import *
import cherrypy

class PwValidatingSchema(validators.Schema):
    linux_pw1 = validators.String(not_empty=True)
    linux_pw2 = validators.String(not_empty=True)
    chained_validators = [validators.FieldsMatch('linux_pw1', 'linux_pw2')]


class ControllerFields(widgets.WidgetsList):
    controller_name = widgets.TextField(label = "Name: ", validator = validators.All(validators.NotEmpty, validators.MinLength(4)), help_text=_(u'* (mind. 4 Zeichen)'))
    desc = widgets.TextArea(label = "Beschreibung:", validator = validators.All(validators.NotEmpty, validators.MinLength(10)), help_text=_(u'* (mind. 10 Zeichen)'))
    linux_user = widgets.TextField(label = "Linux-Benutzer:", validator = validators.All(validators.NotEmpty), help_text=_(u'* Benutzerkonto, unter dem der Controller genutzt wird'))
    linux_pw1 = widgets.PasswordField(validators = validators.All(validators.NotEmpty), label=_(u"Passwort:"), help_text='* Passwort zum Linux-Benutzerkonto')
    linux_pw2 = widgets.PasswordField(validators = validators.All(validators.NotEmpty), label=_(u"Passwort wiederholen:"), help_text='*')
    cmd_init  = widgets.TextField(validators = validators.All(validators.NotEmpty), label=_(u"init-Befehl:"), help_text='* Befehl zum Initialisieren der Umgebung')
    cmd_reset  = widgets.TextField(validators = validators.All(validators.NotEmpty), label=_(u"reset-Befehl:"), help_text=_(u'* Befehl zum Zurcksetzen der Umgebung'))
    cmd_power  = widgets.TextField(validators = validators.All(validators.NotEmpty), label=_(u"power-Befehl:"), help_text=_(u'* Befehl zum Schalten der Versorgungsspannung'))
    available = widgets.CheckBox(label = _(u"Verfgbar?: "), help_text=_(u'Controller fr Benutzer verfgbar?'))

class ReservationFields(widgets.WidgetsList):
    controller_options = [(controller.id, controller.controller_name)
                                    for controller in MicroController.selectBy(available = True)]

    mc = widgets.SingleSelectField(label="Controller:", options = controller_options, validator=validators.String() )
    fromtime = widgets.CalendarDateTimePicker(label = "von:", validator=validators.DateValidator())
    totime = widgets.CalendarDateTimePicker(label = "bis:",validator=validators.DateValidator())



controller_form = widgets.TableForm(fields=ControllerFields(), submit_text = "Controller speichern")
reservation_form = widgets.TableForm(fields=ReservationFields(), submit_text = "reservieren")


controllerinfo =  widgets.DataGrid(fields=[
                                        (u'Controller:', lambda row: row[1]),
                                        (u'Beschreibung:', lambda row: row[2]),
                                    ])

reservationinfo =  widgets.DataGrid(fields=[
                                        (u'Benutzer:', lambda row: row[1]),
                                        (u'von:', lambda row: row[2]),
                                        (u'bis:', lambda row: row[3])
                                    ])

reservationbyuserinfo =  widgets.DataGrid(fields=[
                                        (u'Controller:', lambda row: row[1]),
                                        (u'von:', lambda row: row[2]),
                                        (u'bis:', lambda row: row[3]),
                                        (_(u'lschen?'), lambda row: row[0])
                                    ])

# subcontroller
class MCSubController(controllers.Controller):

    ###############################
    #
    #    controller list
    #
    ###############################
    @expose(template="vlab.templates.mcsubcontroller.index")
    def index(self, *args, **kwargs):
        if identity.in_group("admin"):
            data = [(mc.id, self.makeLink('/controller/details/', mc.id, mc.controller_name), self.shortdesc(mc.desc))
                                        for mc in  MicroController.select()]
        else:
            data = [(mc.id, self.makeLink('/controller/details/', mc.id, mc.controller_name), self.shortdesc(mc.desc))
                                        for mc in  MicroController.selectBy(available = True)]

        return dict(data = data, list = controllerinfo)


    def shortdesc(self, desc):
        if len(desc) >= 50:
            desc = desc[:50] + ' ...'
        return desc

    def makeLink(self, desturl, id, name):
        link = Element('a', href= desturl + '%s' %id)
        link.text = name
        return link

    ###############################
    #
    #     controller details
    #
    ###############################

    @expose(template="vlab.templates.mcsubcontroller.details")
    def details(self, id = None):
        import datetime

        # projectid set

        try:
            mc = MicroController.get(id)
        except:
            id = None


        if id == None:
            flash("Unbekannter Controller!")
            raise redirect('/controller/') #none projectdid => show projectlist

        if mc.available == False and not identity.in_group("admin"):
            flash(u"Controller ist nicht ffentlich!")

            raise redirect('/controller/') #none projectdid => show projectlist
        tabber = widgets.Tabber()

        # get reservations

        self.deleteoldreservations()

        data = [(r.id, self.makeLink('/user/userinfo/',r.user.display_name,r.user.display_name) , r.fromtime, r.totime)
                for r in  Reservation.selectBy(micro_controller = mc)]

        # updateform for admin
        updateform = None
        action = None
        if identity.in_group("admin"):
            updateform = controller_form
            action = "/controller/updatecontroller/%s" %id


        return dict(mc = mc, tabber = tabber, reservationlist = reservationinfo, reservationdata = data,
                    updateform = updateform, action = action)





    ###############################
    #
    #    reservations
    #
    ###############################

    def deleteoldreservations(self):
        """
        delete old reservations
        """
        from datetime import datetime, timedelta

        subselect = Select(Reservation.q.id,  (Reservation.q.totime <= datetime.now()))

        for res in Reservation.select(IN(Reservation.q.id,subselect)):
            res.destroySelf()

        return

    @expose(template="vlab.templates.mcsubcontroller.userreservation")
    def reservation(self):
        self.deleteoldreservations()

        data = [(self.makeLink('/controller/deletereservation/', r.id,_(u'Reservierung lschen')),
                 self.makeLink('/controller/details/', r.micro_controller.id,
                 r.micro_controller.controller_name),
                 r.fromtime, r.totime)
                for r in  Reservation.selectBy(user = identity.current.user)]

        return dict(reservationlist = reservationbyuserinfo, data = data)


    @expose(template="vlab.templates.mcsubcontroller.newreservation")
    @identity.require(identity.Any(identity.not_anonymous()))
    def newreservation(self, *args, **kwargs):
        return dict(reservationform = reservation_form, action='/controller/savereservation')


    @expose()
    @error_handler(newreservation)
    @validate(form=reservation_form)
    @identity.require(identity.Any(identity.not_anonymous()))
    def savereservation(self, *args, **kwargs):
        from datetime import datetime, timedelta
        from sqlobject import *
        from sqlobject.sqlbuilder import *

        fromtime = datetime.strptime(kwargs['fromtime'], "%Y/%m/%d %H:%M")
        totime = datetime.strptime(kwargs['totime'], "%Y/%m/%d %H:%M")

        if totime <= fromtime:
            flash(_(u'Startzeit muss vor Endzeit liegen'))
            raise redirect('/controller/newreservation')
        if fromtime < (datetime.now() + timedelta(minutes= 30)):
            flash(_(u'Statzeit muss nach ' + (datetime.now() + timedelta(minutes= 35)).strftime("%Y-%m-%d %H:%M:%S") + ' liegen'))
            raise redirect('/controller/newreservation')

        if (totime - fromtime) < timedelta(minutes= 15):
            totime = fromtime + timedelta(minutes=15)


        #test overlapping

        #build sql query
        #Visit.q.expiry > datetime.datetime.now())
        # {'fromtime': u'2008/04/09 12:d32', 'mc': u'1', 'totime': u'2008/04/09 12:32'}
        # start bzw ende fllt in einen Reservierungszeitraum ODER Es liegt eine oder mehr reservierungen im Reservierungszeitraum
        selectoverlapping = Select(Reservation.q.id,  OR(
                                                                   AND(Reservation.q.fromtime <= fromtime, Reservation.q.totime >= fromtime),
                                                                   AND(Reservation.q.fromtime <= totime, Reservation.q.totime >= totime),
                                                                   AND(Reservation.q.fromtime >= fromtime, Reservation.q.totime <= totime),

                                                                   )
                                  )

        reservations =  Reservation.select(IN(Reservation.q.id,selectoverlapping))


        if reservations.count() == 0:
            newres = Reservation(user = identity.current.user, micro_controller = MicroController.get(kwargs['mc']) ,
                                 fromtime = fromtime, totime = totime)
        else:
            for res in reservations:
                if res.micro_controller == MicroController.get(kwargs['mc']):
                    flash(_(u'Es liegt bereits eine Reservierung im gewnschten Zeitraum vor!'))
                    raise redirect('/controller/newreservation')

            newres = Reservation(user = identity.current.user, micro_controller = MicroController.get(kwargs['mc']) ,
                             fromtime = fromtime, totime = totime)

        flash('Reservierung eingetragen')


        raise redirect('/controller')

    @expose()
    @identity.require(identity.Any(identity.not_anonymous()))
    def deletereservation(self, id = None):
        try:
            res = Reservation.get(id)
            if res.user == identity.current.user:
                res.destroySelf()
            else:
                flash(_(u'Sie haben nicht die ntigen Rechte, diese Reservierung zu lschen'))
        except:
            flash(_(u'Fehler beim Lschen der Reservierung'))


        raise redirect('/controller/reservation')

    ###############################
    #
    #    create new controller
    #
    ###############################

    @expose(template="vlab.templates.mcsubcontroller.controller")
    @identity.require(identity.Any(identity.not_anonymous()))
    def newcontroller(self, *args, **kwargs):
        action = "/controller/savecontroller/"
        return dict(form = controller_form, action = action)

    @expose()
    @validate(form=controller_form)
    @error_handler(details)
    @identity.require(identity.Any(identity.not_anonymous()))
    def updatecontroller(self, *args, **kwargs):

        c = MicroController.get(args[0])
        kwargs["linux_pw"] = kwargs["linux_pw1"]
        del kwargs["linux_pw1"]
        del kwargs["linux_pw2"]
        c.set(**kwargs)
        raise redirect('/controller/details/%s' %args[0])

    @expose()
    @error_handler(newcontroller)
    @validate(form=controller_form)
    @identity.require(identity.Any(identity.not_anonymous()))
    def savecontroller(self, *args, **kwargs):

        c = MicroController.selectBy(controller_name = kwargs["controller_name"])
        if(c.count() == 0):
            try:
                c = MicroController(controller_name = kwargs["controller_name"],
                                    desc = kwargs["desc"],
                                    linux_user = kwargs["linux_user"],
                                    linux_pw = kwargs["linux_pw2"],
                                    cmd_reset = kwargs["cmd_reset"],
                                    cmd_power = kwargs["cmd_power"],
                                    cmd_init = kwargs["cmd_init"],
                                    available = kwargs["available"])
            except:
                flash('Fehler beim Erstellen des Controllers %s' %kwargs["controller_name"])
                raise redirect('/controller/newcontroller')
        else:
            flash('Es existiert bereits ein Controller %s' %kwargs["controller_name"])
            raise redirect('/controller/newcontroller')
        raise redirect('/controller/details/%s' %c.id)





    ###############################
    #
    #    controller terminal
    #
    ###############################

    @expose(template="vlab.templates.projectsubcontroller.terminal")
    @identity.require(identity.Any(identity.not_anonymous()))
    def use(self, id = None):
        from datetime import datetime
        import sys
        error = False
        projectlist = None
        try:
            mc = MicroController.get(id)
            if mc.available:

                selectoverlapping = Select(Reservation.q.id, where=AND(Reservation.q.fromtime <= datetime.now(),
                                                                       Reservation.q.totime >= datetime.now()

                                                                       ))

                reservations =  Reservation.select(IN(Reservation.q.id,selectoverlapping))

                for res in reservations:
                    if res.user != identity.current.user and res.micro_controller == mc:
                        error = True
                        break

                if error == True:
                    flash(_(u'Der Controller ist bis '+res.totime.strftime("%Y-%m-%d %H:%M:%S") +' durch den Benutzer ' + res.user.display_name + ' reserviert'))
                else:
                     mcu = MicroControllerUse.selectBy(micro_controller = mc)
                     projects = [(p.id, p.projectname) for p in UserProject.selectBy(user = identity.current.user)];
                     projectlist = widgets.SingleSelectField(options = projects, attrs=[('id','project_field')])
                     if mcu.count() >0:
                          if mcu[0].expiry > datetime.now() and mcu[0].user != identity.current.user:
                             error = True
                             flash(_(u'Der Controller wird aktuell von Benutzer '+mcu[0].user.display_name + ' genutzt'))
                          else:
                              mcu[0].destroySelf()




            else:

                flash(_(u'Der gewhlte Controller ist z. Zt. nicht verfgbar!'))
                raise redirect('/controller/details/%s' %id)
        except:

            flash(_(u'Fehler beim Bereitstellen des Controllers'))
            raise redirect('/controller/')

        if error == True:
            raise redirect('/controller')

        """
            setup environment
        """

        cherrypy.session['destroyterm'] = True    # destroy old terminal session
        cherrypy.session['controllerid'] = mc.id

        # editor
        tabber = widgets.Tabber()

        #define attributes for html elemets
        attrs_projectfiles = [('size','25')]
        attrs_editor = [('wrap','off')]
        attrs_output = [('wrap','off'), ('id', 'outputfield')]

        # list of files of project
        projectfiles = widgets.FieldSet(legend="Dateien: ",
                                                     fields = [widgets.SingleSelectField('projectfiles',
                                                                                                            options = [],
                                                                                                            attrs=attrs_projectfiles,
                                                                                                            label='',
                                                                                                            field_class=('projectfilelist'),
                                                                                                            validator = validators.Any)
                                                                ]
                                                    )
        #editor field
        editor = widgets.FieldSet(legend = "Editor: ",
                                  fields = [widgets.TextArea(field_id = "editorfield", attrs=attrs_editor, rows = 24, field_class=('editorfield'))],
                                  )

        output = widgets.FieldSet(legend = "Output: ",
                                  fields = [widgets.TextArea(field_id = "outputfield", attrs=attrs_output, rows = 24, field_class=('editorfield'))],
                                  )

        iconlist = widgets.FieldSet(legend = "Werkzeuge: ",
                                    fields = [widgets.ImageButton(attrs = [("id","newfilebutton")], src = "/static/images/newfile2.png",      field_class=('fieldsethorizontally')),
                                              widgets.ImageButton(attrs = [("id","deletefilebutton")], src = "/static/images/eraser.png",     field_class=('fieldsethorizontally')),
                                              widgets.ImageButton(attrs = [("id","buildbutton")], src = "/static/images/compile.png",     field_class=('fieldsethorizontally')),
                                              #widgets.ImageButton(attrs = [("id","consolebutton")], src = "/static/images/console.png",     field_class=('fieldsethorizontally'))
                                              ]

                                  )

        return dict(projectlist = projectlist, cid = mc.id,
                    tabber = tabber, projectfiles = projectfiles, editor = editor, output = output, iconlist = iconlist)

    """
    following unused
    """


    @expose()
    @identity.require(identity.Any(identity.not_anonymous()))
    def controllerinuse(self,id= None):
        from datetime import datetime, timedelta

        try:
            mc = MicroController.get(id)
            if mc.available:

                selectoverlapping = Select(Reservation.q.id, where=AND(Reservation.q.fromtime <= datetime.now(),
                                                                       Reservation.q.totime >= datetime.now()

                                                                       ))

                reservations =  Reservation.select(IN(Reservation.q.id,selectoverlapping))

                for res in reservations:
                    if res.user != identity.current.user and res.micro_controller == mc:
                        flash(_(u'Der Controller ist bis '+res.totime.strftime("%Y-%m-%d %H:%M:%S")+' durch den Benutzer ' + res.user.display_name + ' reserviert'))
                        raise redirect('/controller/details/' + id)



             #   if reservations.count() > 0:
              #       flash(_(u'Der Controller ist bis '+reservations[0].totime+' durch den Benutzer ' + reservations[0].user.display_name + ' reserviert'))
               #      raise redirect('/controller/details/' + id)
            else:
                flash(_(u'Der gewhlte Controller ist z. Zt. nicht verfgbar!'))
                raise redirect('/controller/details/' + id)
        except:
            flash(_(u'Fehler beim Bereitstellen der Controller-Informationen'))
            raise redirect('/controller/')

        return dict(available = True)



