#!/usr/bin/python
# -*- coding: latin-1 -*-

# Copyright (c) 2008 by Bernd Kalwar
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA

from turbogears import controllers, expose, widgets, flash, validators, redirect, identity, error_handler, validate
from sqlobject import *
from kid import Element
from vlab.model import User, UserProject, MicroController as VlabController
import cherrypy, os

PROJECT_DIR= cherrypy.config.get("vlab.projectdir")

class NewProjectFields(widgets.WidgetsList):
    #from vlab.model import ProgrammingLanguage
    #language_options = [(pl.id, pl.language)
    #                        for pl in ProgrammingLanguage.select()]
    #language_options.append((-1, 'n/a'))
    #projecttableform = widgets.TableForm()
    projectname = widgets.TextField(label = "Projekt-Name: ", validator = validators.All(validators.NotEmpty, validators.PlainText,  validators.MaxLength(50), validators.MinLength(5)), help_text=_(u'* (5-50 Zeichen)'))
    #language = widgets.SingleSelectField(label="Programmier-Sprache:",validator=validators.Any, options = language_options)
    desc = widgets.TextArea(label = "Beschreibung:", validator = validators.All(validators.NotEmpty, validators.MinLength(10)), help_text=_(u'* (mind. 10 Zeichen)'))
    public = widgets.CheckBox(label = _(u"ffentlich?: "))

class CopyProjectFields(NewProjectFields, widgets.WidgetsList):
    projectname = widgets.TextField(label = "Projekt-Name: ", validator = validators.All(validators.NotEmpty, validators.PlainText,  validators.MaxLength(50), validators.MinLength(5)), help_text=_(u'* (5-50 Zeichen)'))
    desc = widgets.TextArea(label = "Beschreibung:", validator = validators.All(validators.NotEmpty, validators.MinLength(10)), help_text=_(u'* (mind. 10 Zeichen)'))
    public = widgets.CheckBox(label = _(u"ffentlich?: "))
    sourceprojectid = widgets.HiddenField(name = "sourceprojectid")

newproject_form = widgets.TableForm(fields = NewProjectFields(), submit_text = "Projekt erstellen")
updateproject_form = widgets.TableForm(fields = NewProjectFields(), submit_text = "speichern")
copyproject_form = widgets.TableForm(fields = CopyProjectFields(), submit_text = "Kopie erstellen")

class ProjectFilterFields(widgets.WidgetsList):

    project  = widgets.TextField(label = "Projekt: ")
    #language_options = [(pl.id, pl.language)
   #                         for pl in ProgrammingLanguage.select()]
    #language_options.append((-1, 'alle'))

   # language = widgets.SingleSelectField(label="Programmier-Sprache: ",validator=validators.Any, options = language_options)
    user = widgets.TextField(label = "Benutzername: ")



projectinfo =  widgets.DataGrid(fields=[
                                        (u'Projekt:', lambda row: row[2]),
                                        (u'Beschreibung:', lambda row: row[3]),

                                        (u'Benutzer:', lambda row: row[1])
                                    ])


# user subcontroller
class ProjectSubController(controllers.Controller):


    ###############################
    #
    #    project list
    #
    ###############################

    @expose(template="vlab.templates.projectsubcontroller.index")
    def index(self, *args, **kwargs):
        project_searchform = widgets.RemoteForm(
                                     name="ItemSearch",
                                     fields=ProjectFilterFields(),
                                     submit_text="filtern")

        filterfields = widgets.FieldSet(legend="Filter: ",
                                                fields = project_searchform
                                                    )

        data = [(project.id, self.makeLink('/user/userinfo/', project.user.display_name, project.user.display_name), self.makeLink('/projects/details/', project.id, project.projectname), self.shortdesc(project.desc))
                                        for project in  UserProject.selectBy(public = True)]

        return dict(projectlist = projectinfo, project_searchform = project_searchform,  data = data, filterfields = filterfields)



    @expose(template="vlab.templates.projectsubcontroller.index")
    @identity.require(identity.Any(identity.not_anonymous()))
    def myprojects(self):
        data = [(project.id, self.makeLink('/user/userinfo/', project.user.display_name, project.user.display_name), self.makeLink('/projects/details/', project.id, project.projectname), self.shortdesc(project.desc))
                                        for project in  UserProject.selectBy(user = identity.current.user)]

        return dict(projectlist = projectinfo,data = data, project_searchform = None)


    def shortdesc(self, desc):
        if len(desc) >= 50:
            desc = desc[:50] + ' ...'
        return desc

    def makeProjectLink(self, projectid, projectname):
        link = Element('a', href='/projects/details/%s' % projectid)
        link.text = projectname
        return link

    def makeLink(self, desturl, id, name):
        link = Element('a', href= desturl + '%s' %id)
        link.text = name
        return link

    @expose()
    def searchproject(self, *args, **kwargs):
        from sqlobject.sqlbuilder import EXISTS, Select, Outer

        data = [(project.id, self.makeLink('/user/userinfo/', project.user.display_name, project.user.display_name), self.makeLink('/projects/details/', project.id, project.projectname), self.shortdesc(project.desc))
                                    for project in UserProject.select(
                                                                      AND(LIKE(UserProject.q.projectname, "%" + kwargs['project'] + "%"),
                                                                          UserProject.q.public == True)
                                                                      ) ]

        # return renderd data
        return projectinfo.render(data)



    @expose()
    def download(self, *args, **kwargs):
        import cherrypy
        from cherrypy.lib.cptools import serveFile

        try:
            project = UserProject.get(kwargs["projectid"])
            target = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))
            filename = project.projectname + '.tar'

            os.chdir(target)

            target = os.path.join(target,'tar')
            if not os.path.exists(target):
                os.makedirs(target)

            # build the tar file

        #  tar -cvvf  tmp.tar --no-recursion ./
            os.system('tar -czf ./tar/'+filename + ' --no-recursion *')
            filepath = os.path.join(target, filename)

            return serveFile(filepath, "application/x-download", "attachment")
        except:
            flash("Fehler beim Bereitstellen des Downloads")

        return dict()



    ###############################
    #
    #    new project
    #
    ###############################

    @expose(template="vlab.templates.projectsubcontroller.newproject")
    @identity.require(identity.Any(identity.not_anonymous()))
    def newproject(self):
        action = "/projects/saveproject/"
        return dict(form = newproject_form, action = action)

    @expose()
    @error_handler(newproject)
    @validate(form=newproject_form)
    @identity.require(identity.Any(identity.not_anonymous()))
    def saveproject(self, *args, **kwargs):
        if args and args[0] == "update":
            p = UserProject.get(args[1])
            p.set(**kwargs)
            raise redirect('/projects/details/%s' %p.id)
        else:
            public = False
            if kwargs.has_key('public'):
                public = kwargs['public']

            p = UserProject.selectBy(user = identity.current.user, projectname = kwargs["projectname"])
            if(p.count() == 0):
                try:
                    p = UserProject(user = identity.current.user, owner = identity.current.user, projectname = kwargs["projectname"],
                                    desc = kwargs["desc"], public = public)
                except:
                    flash('Fehler beim Erstellen des Projektes %s' %kwargs["projectname"])
                    raise redirect('/projects/newproject')
            else:
                flash('Es existiert bereits ein Projekt %s' %kwargs["projectname"])
                raise redirect('/projects/newproject')
            raise redirect('/projects/details/%s' %p.id)




    ###############################
    #
    #    project details and options
    #
    ###############################

    @expose(template="vlab.templates.projectsubcontroller.details")
    def details(self, projectid = None):
        # projectid set
        try:
            project = UserProject.get(projectid)
        except:
            projectid = None


        if projectid == None:
            flash("Unbekanntes Projekt!")
            raise redirect('/projects/') #none projectdid => show projectlist

        if project.public == False and not(project.user == identity.current.user or identity.in_group("admin")):
            flash(u"Projekt ist nicht ffentlich!")

            raise redirect('/projects/') #none projectdid => show projectlist

        #define attributes for html elemets
        attrs_projectfiles = [('size','25')]
        attrs_editor = [('wrap','off')]
        attrs_output = [('wrap','off'), ('id', 'outputfield')]

        # list of files of project
        projectfiles = widgets.FieldSet(legend="Dateien: ",
                                                     fields = [widgets.SingleSelectField('projectfiles',
                                                                                                            options = [],
                                                                                                            attrs=attrs_projectfiles,
                                                                                                            label='',
                                                                                                            field_class=('projectfilelist'),
                                                                                                            validator = validators.Any)
                                                                ]
                                                    )
        #editor field
        editor = widgets.FieldSet(legend = "Editor: ",
                                  fields = [widgets.TextArea(field_id = "editorfield", attrs=attrs_editor, rows = 24, field_class=('editorfield'))],
                                  )

        output = widgets.FieldSet(legend = "Output: ",
                                  fields = [widgets.TextArea(field_id = "outputfield", attrs=attrs_output, rows = 24, field_class=('editorfield'))],
                                  )

        # icons


        iconlist = widgets.FieldSet(legend = "Werkzeuge: ",
                                    fields = [widgets.ImageButton(attrs = [("id","newfilebutton")], src = "/static/images/newfile2.png",      field_class=('fieldsethorizontally')),
                                              widgets.ImageButton(attrs = [("id","deletefilebutton")], src = "/static/images/eraser.png",     field_class=('fieldsethorizontally')),
                                              widgets.ImageButton(attrs = [("id","buildbutton")], src = "/static/images/compile.png",     field_class=('fieldsethorizontally')),
                                              #widgets.ImageButton(attrs = [("id","consolebutton")], src = "/static/images/console.png",     field_class=('fieldsethorizontally'))
                                              ]

                                  )

        # option field
        #project settins
        settingsform = None
        download = None
        if (project.user == identity.current) or ("admin" in identity.current.groups):
               settingsform =  widgets.RemoteForm(
                                                  name="ItemSearch",
                                                  fields=NewProjectFields(),
                                                  submit_text="speichern")

        dllink = Element('a', href='/projects/download?projectid=%s' % projectid)
        dllink.text = "Projekt herunterladen"



        tabber = widgets.Tabber()
        updateform = None;
        action = ""
        if project.user == identity.current.user :
            updateform = updateproject_form
            action = "/projects/saveproject/update/%s" %project.id


        return dict(output = output, updateform = updateform, action = action,  tabber = tabber, project = project, projectfiles = projectfiles, editor = editor, iconlist = iconlist, settings = settingsform)




    ###############################
    #
    #    delete project
    #
    ###############################
    @expose()
    @identity.require(identity.Any(identity.not_anonymous()))
    def delete(self, projectid = None):
        try:
            project = UserProject.get(projectid)
            target_dir = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))

            # remove files and subdirs in folder
            for root, dirs, files in os.walk(target_dir, topdown=False):

                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))

            #remove project folder
            os.rmdir(target_dir)

            #remove project from database
            if (project.user == identity.current.user) or ("admin" in identity.current.groups):
                project.destroySelf()

        except:
            flash('Projekt existiert nicht!')

        if project.user == identity.current.user:
            flash(_(u"Projekt %s wurde erfolgreich gelscht" % project.projectname))
        else:
            flash(_(u"Sie besitzten nicht die erforerlichen Rechte, das Projekt zu lschen!"))
        raise redirect('/projects')

    ###############################
    #
    #    add files to project
    #
    ###############################

    @expose('json')
    def checkfileexists(self, *args, **kwargs):

        exists = True

        try:
            project = UserProject.get(kwargs["projectid"])
            if (project.user == identity.current.user) or ("admin" in identity.current.groups):

                target_file_name =  os.path.join(PROJECT_DIR, project.user.display_name,str(project.id), kwargs["filename"])

                target_dir = os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))

                if not os.path.exists(target_dir):
                    os.makedirs(target_dir)

                if not os.path.isfile(target_file_name):
                    exists = False

        except:
            return dict(exists = True, filename = kwargs['filename']);

        return dict(exists = exists, filename = kwargs['filename']);


    @expose(template="vlab.templates.projectsubcontroller.upload")
    @identity.require(identity.Any(identity.not_anonymous()))
    def uploadfile(self, projectid = None):
        if projectid != None:
            try:
                project = UserProject.get(projectid)
                projectid = widgets.HiddenField(name = "projectid", default = projectid)
                projectfile = widgets.FileField(name = "projectfile", label='Datei: ', help_text=_(u'*.tar.gz, *.zip werden automatisch entpackt'))
                upload_form = widgets.TableForm(name = "Upload", fields = [projectfile,projectid], submit_text = "Datei hochladen", action='/projects/upload')
                return dict(uploadform = upload_form, project = project)
            except:
                pass
        else:
            pass

        flash('Projekt nicht vorhanden')
        raise redirect('/projects')


    @expose()
    @identity.require(identity.Any(identity.not_anonymous()))
    def upload(self, *args, **kwargs):
        projectfile = kwargs['projectfile']
        projectid = kwargs['projectid']
        data = projectfile.file.read()
        project = UserProject.get(projectid)
        target  =os.path.join(PROJECT_DIR, project.user.display_name,str(project.id))
        target_file_name =  os.path.join(PROJECT_DIR, project.user.display_name,str(project.id), kwargs['projectfile'].filename)
        if not os.path.exists(target):
                os.makedirs(target)

        # zippen file?
        file_extension =  projectfile.filename.split('.')
        file_extension = projectfile.filename.split(file_extension[0])
        file_extension = file_extension[len(file_extension)-1]

        zipcommands = {".tar":"tar xfv ",".tar.gz":"tar xfvz " }

        zipoptions = {"tar":" -k","tar.gz":" -k"}


        f = open(target_file_name, 'w')
        f.write(data)
        f.close()
        if file_extension in zipcommands:
            #it's a zipped file -> unzip it
            os.chdir(target)
            cmd = zipcommands[file_extension] + target_file_name
            if file_extension in zipoptions:
                cmd = cmd + zipoptions[file_extension]
            os.system(cmd)
            os.remove(target_file_name)
            # delete zip file

        raise redirect('/projects/details/%s' % projectid)

    ###############################
    #
    #    copy project to userspace
    #
    ###############################
    @expose(template="vlab.templates.projectsubcontroller.copy")
    @identity.require(identity.Any(identity.not_anonymous()))
    def copy(self, projectid = None):
        if projectid == None:
            flash("Unbekanntes Projekt!")
            raise redirect('/projects')

        try:
            project = UserProject.get(projectid)
            if project.public == False:
                flash("Projekt ist nicht ffentlich!")
                raise redirect('/projects')

            values = {"sourceprojectid":project.id}
            return dict(project = project, copyproject_form = copyproject_form, values = values,action = '/projects/copyproject')


        except:
            flash("Fehler beim Laden des Quell-Projekts!")
            raise redirect('/projects')


    @expose()
    @error_handler(copy)
    @validate(form=copyproject_form)
    @identity.require(identity.Any(identity.not_anonymous()))
    def copyproject(self, *args, **kwargs):

        import shutil
        public = False
        if kwargs.has_key('public'):
            public = kwargs['public']

        p = UserProject.selectBy(user = identity.current.user, projectname = kwargs["projectname"])
        sourceproject = UserProject.get(kwargs['sourceprojectid'])
        if(p.count() == 0):
            try:
                p = UserProject(user = identity.current.user, owner = sourceproject.user, projectname = kwargs["projectname"],
                                desc = kwargs["desc"], public = public)
                target_dir = os.path.join(PROJECT_DIR, p.user.display_name,str(p.id))
                source_dir = os.path.join(PROJECT_DIR, sourceproject.user.display_name,str(sourceproject.id))
                shutil.copytree(source_dir, target_dir)

            except:
                flash('Fehler beim Erstellen des Projektes %s' %kwargs["projectname"])
                raise redirect('/projects/copy?projectid=%s' %sourceproject.id)
        else:
            flash('Es existiert bereits ein Projekt %s' %kwargs["projectname"])
            raise redirect('/projects/copy?projectid=%s' %sourceproject.id)

        raise redirect('/projects/details/%s' %p.id)

