#!/usr/bin/python
# -*- coding: latin-1 -*-

# Copyright (c) 2008 by Bernd Kalwar
#
# GNU General Public Licence (GPL)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA

from turbogears import controllers, expose, widgets, flash, validators,validate, identity, redirect, error_handler
from kid import Element
from sqlobject import *
from vlab.model import User, UserProject


class PwValidatingSchema(validators.Schema):
    pwd1 = validators.String(not_empty=True)
    pwd2 = validators.String(not_empty=True)
    chained_validators = [validators.FieldsMatch('pwd1', 'pwd2')]


class SignupFormFields(widgets.WidgetsList):
    username = widgets.TextField(validator = validators.All(validators.NotEmpty, validators.PlainText,  validators.MaxLength(20), validators.MinLength(5)), label = _(u'Gewnschter Benutzername:'),help_text=_(u'* (5-20 Zeichen)'))
    pwd1 = widgets.PasswordField(validators = validators.All(validators.NotEmpty), label=_(u"Passwort:"), help_text='*')
    pwd2 = widgets.PasswordField(validators = validators.All(validators.NotEmpty), label=_(u"Passwort wiederholen:"),help_text='*')
    email = widgets.TextField(validator = validators.All(validators.Email, validators.NotEmpty), label = _(u"E-Mail:"),help_text='*')
    firstname = widgets.TextField(validator = validators.All(validators.NotEmpty, validators.MaxLength(50)),label = _(u"Vorname:"),help_text='*')
    lastname = widgets.TextField(validator = validators.All(validators.NotEmpty,validators.MaxLength(50)), label = _(u"Nachname:"),help_text='*')
    zipcode = widgets.TextField(validator = validators.All(validators.MaxLength(5)),label = _(u"PLZ:"))
    city = widgets.TextField(label = _(u"Ort:"))

class UserUpdateFormFields(widgets.WidgetsList):
    pwd1 = widgets.PasswordField(validators = validators.All(validators.NotEmpty), label=_(u"Passwort:"), help_text='*')
    pwd2 = widgets.PasswordField(validators = validators.All(validators.NotEmpty), label=_(u"Passwort wiederholen:"),help_text='*')
    email = widgets.TextField(validator = validators.All(validators.Email, validators.NotEmpty), label = _(u"E-Mail:"),help_text='*')
    zipcode = widgets.TextField(validator = validators.All(validators.MaxLength(5)),label = _(u"PLZ:"))
    city = widgets.TextField(label = _(u"Ort:"))

signup_form = widgets.TableForm(fields = SignupFormFields(), submit_text = _(u"Profil erstellen"), validator= PwValidatingSchema())
update_form = widgets.TableForm(fields = UserUpdateFormFields(), submit_text = _(u"Profil speichern"), validator= PwValidatingSchema())

class UserProjectFilterFields(widgets.WidgetsList):
   # from vlab.model import ProgrammingLanguage
   # language_options = [(pl.id, pl.language)
    #                        for pl in ProgrammingLanguage.select()]

   # controller = widgets.SingleSelectField(label="Programmier-Sprache:",validator=validators.Any, options = language_options)

    submit_button = widgets.ImageButton(label = '', src = "/static/images/ok.png")


# user subcontroller
class UserSubController(controllers.Controller):

    @expose(template="vlab.templates.usersubcontroller.alluser")
    @expose(format="json")
    def index(self, letter = None):
        userinfo =  widgets.DataGrid(fields=[
                    ('Benutzer', lambda row: row[1]),
                    ('angemeldet seit:', lambda row: row[2]),
                ])

        if letter == None:
            data = [(user.id, self.makeUsernameLink(user.user_name), user.created)
                                    for user in  User.select()]
        else:
            data = [(user.id, self.makeUsernameLink(user.user_name), user.created)
                                    for user in  User.select(User.q.user_name.startswith(letter))]


        return dict(usergrid = userinfo, data = data)


    def userdetail(self, user ):
        import sys
        try:
            userdict = dict(id = user.id,  username = user.user_name, surname=user.lastname, firstname = user.firstname, zipcode = user.zipcode, city = user.city, created = user.created)
        except:
            print "Unexpected error:", sys.exc_info()[0]
        return userdict

    @expose()
    def currentuserdetails(self):
        return dict()

    def makeLink(self, desturl, id, name):
        link = Element('a', href= desturl + '%s' %id)
        link.text = name
        return link

    def makeUsernameLink(self, username):
        link = Element('a', href='/user/userinfo/%s' % username)
        link.text = username
        return link

    @expose(template="vlab.templates.usersubcontroller.userinfo")
    def userinfo(self, username = None):

        from sqlobject import SQLObjectNotFound
        import sys
        userdetails = None
        userprojects = None
        data = None
        error = None
        filterfields = None
        try:
            # select user
            users = User.selectBy(user_name = username)
            user = users[0]

            userdetails = self.userdetail(user = user)

        except:
            flash("FEHLER: Benutzer %s nicht im System vorhanden!" %username)
            error = 1

        if error == None:
            try:

                filterfields = widgets.FieldSet(legend="Filter: ",
                                                fields = UserProjectFilterFields()
                                                    )

                data = [(self.makeLink('/projects/details/', project.id, project.projectname), project.desc, project.public)
                                    for project in  UserProject.selectBy(user = user)]



                userprojects =  widgets.DataGrid(fields=[
                    ('Projekt', lambda row: row[0]),
                    ('Beschreibung', lambda row: row[1]),
                    (_(u'ffentlich'), lambda row: row[2])
                ])

            except:
                flash("FEHLER: Problem beim Abrufen der Projektdaten von Benutzer %s" %username)

        tabber = widgets.Tabber()
        submitaction = "/user/newuser/update/%s" %user.id

        return dict(tabber = tabber, user = user, action = submitaction, form = update_form, userdetails = userdetails, userprojects = userprojects, data = data, filterfields = filterfields)


    ###############################
    #
    #    User management
    #
    ###############################


    @expose()
    @identity.require(identity.Any(identity.not_anonymous()))
    def deleteuser(self, userid = None):
        """

        """
        try:
            if userid == None:
                pass
            else:
                if (identity.current == User.get(userid)) or ("admin" in identity.current.groups):
                    User.delete(userid)
                else:
                    raise identity.current.GroupMembershipRequiredException(("admin"))
        except:
            flash('Fehler beim Loeschen des Benutzers')

        raise redirect('/user/')


    @expose(template="vlab.templates.usersubcontroller.signup")
    def signup(self):

        """
        returns a signup form if current user is not logged in, otherwise redirect to main page
        """
        if identity.current.anonymous:
            submit_action = "/user/newuser"
            return dict(form = signup_form, action = submit_action)
        raise redirect('/')


    @expose()
    @error_handler(signup)
    @validate(form=signup_form)
    def newuser(self, *args,**kwargs):
        if args and args[0] == "update":
            u = User.get(args[1])
            raise redirect('/user/userinfo/%s' %u.user_name)
        else:
            try:
                u = User(user_name = kwargs['username'], display_name = kwargs['username'],
                         email_address = kwargs['email'], password = kwargs['pwd1'],
                         city = kwargs['city'], zipcode = kwargs['zipcode'],
                         firstname = kwargs['firstname'], lastname = kwargs['lastname']
                         )
            except:
                flash('Benutzername oder E-Mail-Adresse wird bereits verwendet!')
                raise redirect('/user/signup')


            raise redirect('/')
